﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.IO;
using System.Text;


using System.ServiceModel;
using System.ServiceModel.Description;

using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using Microsoft.Xrm.Sdk.Client;
using Microsoft.Xrm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Metadata;

using Microsoft.Xrm.Client;
using Microsoft.Xrm.Client.Services;

using Microsoft.Crm.Sdk;
using Microsoft.Crm.Sdk.Messages;

public class NotificationCodeForCrm
{
	static string apikey = System.Configuration.ConfigurationManager.AppSettings["ChatApiKey"];

	static CrmConnection _crmconn;
	static object _crmconnlock = new object();

	public static OrganizationService OpenCrmSvc()
	{
		lock (_crmconnlock)
		{
			if (_crmconn == null)
			{
				System.Configuration.ConnectionStringSettings css = new System.Configuration.ConnectionStringSettings();
				css.Name = "Crm";
				css.ConnectionString = System.Configuration.ConfigurationManager.AppSettings["CrmConnectionString"];
				CrmConnection conn = new CrmConnection(css);
				conn.ClientCredentials = new ClientCredentials();
				conn.ClientCredentials.UserName.UserName = System.Configuration.ConfigurationManager.AppSettings["CrmUserName"];
				conn.ClientCredentials.UserName.Password = System.Configuration.ConfigurationManager.AppSettings["CrmPassWord"];

				_crmconn = conn;
			}
		}

		return new OrganizationService(_crmconn);
	}


	static public bool HandleChatNotification(RequestData rd, TextWriter log)
	{
		switch (rd.type)
		{
			case "AGENT_ONLINE":	//AGENT LOGIN CONSOLE			args=agent,channel
				break;
			case "AGENT_OFFLINE":	//AGENT CLOSE CONSOLE			args=agent,channel
				break;
			case "VISITOR_ONLINE":	//VISITOR ENTER WEBSITE			args=visitor,channel
				break;
			case "VISITOR_OFFLINE":	//VISITOR LEAVE WEBSITE			args=visitor,channel
				break;

			case "AGENT_ENTER":	//AGENT ENTER CHAT ROOM				args=agent,channel,visitor
				{
					ChatApiNS.ChatApiWebService sc = new ChatApiNS.ChatApiWebService();
					sc.Url = System.Configuration.ConfigurationManager.AppSettings["ChatApiServiceUrl"];

					string message = "Welcome , click more information about this visitor : " + (string)rd.args[2]["DisplayName"];
					sc.SendAnnouncement(apikey, (string)rd.args[1]["ChannelName"], (string)rd.args[0]["UserId"], message,
						"<a href='http:///' target='_blank' style='color:red'>" + System.Web.HttpUtility.HtmlEncode(message) + "</a>"
						);
				}
				break;
			case "AGENT_LEAVE":	//AGENT LEAVE CHAT ROOM				args=agent,channel,visitor
				break;
			case "VISITOR_ENTER":	//VISITOR ENTER CHAT ROOM		args=chatter,channel,visitor
				{
					ChatApiNS.ChatApiWebService sc = new ChatApiNS.ChatApiWebService();
					sc.Url = System.Configuration.ConfigurationManager.AppSettings["ChatApiServiceUrl"];

					string message = "Welcome , this message is sent from api test website.";
					sc.SendAnnouncement(apikey, (string)rd.args[1]["ChannelName"], (string)rd.args[0]["UserId"], message,
						"<span style='color:red'>" + System.Web.HttpUtility.HtmlEncode(message) + "</span>"
						);
				}
				break;
			case "VISITOR_LEAVE":	//VISITOR LEAVE CHAT ROOM		args=chatter,channel,visitor
				break;

			case "OFFLINEMESSAGE_SEND":	//							args=msgdata,visitor
				EnsureContact(rd.args[1], rd.type, rd.args[0]);
				break;

			case "MAILTEMPLATE_SEND":	//							args=msgdata,moredata
				break;

			case "VISITOR_WAITCHAT":	//							args=visitor
				EnsureContact(rd.args[0], rd.type);
				break;
			case "VISITOR_MISSCHAT":	//							args=visitor
				EnsureContact(rd.args[0], rd.type);
				break;
			case "VISITOR_BEACCEPTED":	//							args=visitor
				EnsureContact(rd.args[0], rd.type);
				break;

			case "CONVERSATION_END":	//							args=visitor,channel
				EnsureContact(rd.args[0], rd.type, rd.args[1]);
				{
					ChatApiNS.ChatApiWebService sc = new ChatApiNS.ChatApiWebService();
					sc.Url = System.Configuration.ConfigurationManager.AppSettings["ChatApiServiceUrl"];
					object sid;
					if (rd.args[1].TryGetValue("SessionId", out sid))
					{
						ChatApiNS.ChatMsgData[] msgs = sc.LoadSessionMessages(apikey, Convert.ToInt32(sid));
						log.WriteLine(" Messages : " + msgs.Length);
						foreach (ChatApiNS.ChatMsgData msg in msgs)
						{
							log.WriteLine(msg.Sender + " : " + msg.Text);
						}
					}
				}
				break;
			case "CONVERSATION_START":	//							args=visitor,channel
				EnsureContact(rd.args[0], rd.type, rd.args[1]);
				break;

			case "CONVERSATION_MESSAGE":	//						args=msgdata,visitor,channel
				{
					ChatApiNS.ChatApiWebService sc = new ChatApiNS.ChatApiWebService();
					sc.Url = System.Configuration.ConfigurationManager.AppSettings["ChatApiServiceUrl"];

					string message = "Test......";
					sc.SendAnnouncement(apikey, (string)rd.args[2]["ChannelName"], (string)rd.args[1]["UserId"], message,
						"<span style='color:red'>" + System.Web.HttpUtility.HtmlEncode(message) + "</span>"
						);
				}
				break;
		}

		return true;
	}

	static T GetVal<T>(Dictionary<string, object> dict, string key)
	{
		object val;
		if (!dict.TryGetValue(key, out val))
			return default(T);
		return (T)Convert.ChangeType(val, typeof(T));
	}

	static Contact EnsureContact(Dictionary<string, object> visitor, string eventname, Dictionary<string, object> moredata = null)
	{
		Contact contact;
		string dispname = GetVal<string>(visitor, "DisplayName");
		string email = GetVal<string>(visitor, "Email");

		System.Web.Script.Serialization.JavaScriptSerializer jss = new System.Web.Script.Serialization.JavaScriptSerializer();

		using (OrganizationService svc = OpenCrmSvc())
		{
			RetrieveMultipleRequest rmr = new RetrieveMultipleRequest();
			QueryExpression qe = new QueryExpression()
				{
					EntityName = "contact"
					,
					ColumnSet = new ColumnSet(true)
				};
			qe.Criteria.AddCondition(new ConditionExpression("emailaddress1", ConditionOperator.Equal, email));
			rmr.Query = qe;
			RetrieveMultipleResponse rmres = svc.RetrieveMultiple(rmr);
			if (rmres.EntityCollection.Entities.Count != 0)
			{
				contact = (Contact)rmres.EntityCollection.Entities[0];
				//do not update the contact name until the source is trust-able
			}
			else
			{
				contact = new Contact();
				contact.NickName = dispname;
				contact.FirstName = dispname;
				contact.LastName = "";
				contact.EMailAddress1 = email;
				contact.Description = "@CHATAPI";
				contact.Id = svc.Create(contact);

				Annotation note = new Annotation();
				note.ObjectId = contact.ToEntityReference();
				note.NoteText = "CONTACT CREATED : " + jss.Serialize(visitor);
				svc.Create(note);
			}

			{
				Annotation note = new Annotation();
				note.ObjectId = contact.ToEntityReference();
				note.NoteText = eventname + " : " + jss.Serialize(visitor);

				if (moredata != null)
					note.NoteText += "\r\n" + jss.Serialize(moredata);

				svc.Create(note);
			}

			return contact;
		}



	}
}
